@@ -1,8 +1,11 @@
 # frozen_string_literal: true
 
+require_relative "force_platform"
+
 module Bundler
   class LazySpecification
     include MatchPlatform
+    include ForcePlatform
 
     attr_reader :name, :version, :dependencies, :platform
     attr_accessor :source, :remote, :force_ruby_platform
@@ -13,6 +16,7 @@ module Bundler
       @dependencies  = []
       @platform      = platform || Gem::Platform::RUBY
       @source        = source
+      @force_ruby_platform = default_force_ruby_platform
     end
 
     def full_name
@@ -107,7 +111,7 @@ module Bundler
       search = candidates.reverse.find do |spec|
         spec.is_a?(StubSpecification) ||
           (spec.matches_current_ruby? &&
-           spec.matches_current_rubygems?)
+            spec.matches_current_rubygems?)
       end
       if search.nil? && fallback_to_non_installable
         search = candidates.last
