@@ -9,28 +9,28 @@
 # lost the next time ruby is built.
 
 module RbConfig
-  RUBY_VERSION.start_with?("3.1.") or
-    raise "ruby lib version (3.1.6) doesn't match executable version (#{RUBY_VERSION})"
+  RUBY_VERSION.start_with?("3.2.") or
+    raise "ruby lib version (3.2.1) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
-  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.1/sparc64-openbsd")
+  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.2/sparc64-openbsd")
   # DESTDIR on make install.
   DESTDIR = '' unless defined? DESTDIR
   # The hash configurations stored.
   CONFIG = {}
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
-  CONFIG["MINOR"] = "1"
-  CONFIG["TEENY"] = "6"
-  CONFIG["PATCHLEVEL"] = "260"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.1.6/bin/install -c'
+  CONFIG["MINOR"] = "2"
+  CONFIG["TEENY"] = "1"
+  CONFIG["PATCHLEVEL"] = "31"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.2.1/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
-  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)31"
-  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)31"
-  CONFIG["RUBY_SO_NAME"] = "ruby31"
+  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)32"
+  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)32"
+  CONFIG["RUBY_SO_NAME"] = "ruby32"
   CONFIG["exec"] = "exec"
-  CONFIG["ruby_pc"] = "ruby-3.1.pc"
+  CONFIG["ruby_pc"] = "ruby-3.2.pc"
   CONFIG["CC_WRAPPER"] = ""
   CONFIG["PACKAGE"] = "ruby"
   CONFIG["BUILTIN_TRANSSRCS"] = " enc/trans/newline.c"
@@ -45,7 +45,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=31' '--with-soname=ruby31' '--with-ruby-version=3.1' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--with-compress-debug-sections=no' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=32' '--with-soname=ruby32' '--with-ruby-version=3.2' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -55,7 +55,7 @@ module RbConfig
   CONFIG["sitedir"] = "$(rubylibprefix)/site_ruby"
   CONFIG["rubyarchdir"] = "$(rubylibdir)/$(arch)"
   CONFIG["rubylibdir"] = "$(rubylibprefix)/$(ruby_version)"
-  CONFIG["ruby_version"] = "3.1"
+  CONFIG["ruby_version"] = "3.2"
   CONFIG["sitearch"] = "$(arch)"
   CONFIG["arch"] = "sparc64-openbsd"
   CONFIG["sitearchincludedir"] = "$(includedir)/$(sitearch)"
@@ -64,7 +64,7 @@ module RbConfig
   CONFIG["archlibdir"] = "$(libdir)/$(arch)"
   CONFIG["libdirname"] = "libdir"
   CONFIG["RUBY_EXEC_PREFIX"] = "/usr/local"
-  CONFIG["RUBY_LIB_VERSION"] = "3.1"
+  CONFIG["RUBY_LIB_VERSION"] = "3.2"
   CONFIG["RUBY_LIB_VERSION_STYLE"] = ""
   CONFIG["RI_BASE_NAME"] = "ri"
   CONFIG["ridir"] = "$(datarootdir)/$(RI_BASE_NAME)"
@@ -72,13 +72,14 @@ module RbConfig
   CONFIG["rubyarchprefix"] = "$(rubylibprefix)/$(arch)"
   CONFIG["MAKEFILES"] = "Makefile"
   CONFIG["PLATFORM_DIR"] = ""
+  CONFIG["COROUTINE_TYPE"] = "pthread"
   CONFIG["THREAD_MODEL"] = "pthread"
   CONFIG["SYMBOL_PREFIX"] = ""
   CONFIG["EXPORT_PREFIX"] = ""
   CONFIG["COMMON_HEADERS"] = ""
   CONFIG["COMMON_MACROS"] = ""
   CONFIG["COMMON_LIBS"] = ""
-  CONFIG["MAINLIBS"] = "-lz -pthread -lgmp -lm "
+  CONFIG["MAINLIBS"] = "-lz -lgmp -lm -pthread "
   CONFIG["ENABLE_SHARED"] = "yes"
   CONFIG["DLDSHARED"] = "$(CC) -shared $(CCDLFLAGS)"
   CONFIG["DLDLIBS"] = "-lc"
@@ -95,14 +96,20 @@ module RbConfig
   CONFIG["rubyw_install_name"] = ""
   CONFIG["EXTDLDFLAGS"] = ""
   CONFIG["EXTLDFLAGS"] = ""
-  CONFIG["strict_warnflags"] = "-std=gnu99"
-  CONFIG["warnflags"] = "-Wall -Wextra -Wdeprecated-declarations -Wimplicit-function-declaration -Wimplicit-int -Wpointer-arith -Wwrite-strings -Wold-style-definition -Wmissing-noreturn -Wno-cast-function-type -Wno-constant-logical-operand -Wno-long-long -Wno-missing-field-initializers -Wno-overlength-strings -Wno-packed-bitfield-compat -Wno-parentheses-equality -Wno-self-assign -Wno-tautological-compare -Wno-unused-parameter -Wno-unused-value -Wunused-variable -Wundef"
+  CONFIG["strict_warnflags"] = ""
+  CONFIG["warnflags"] = "-Wall -Wextra -Wdeprecated-declarations -Wdiv-by-zero -Wduplicated-cond -Wimplicit-function-declaration -Wimplicit-int -Wmisleading-indentation -Wpointer-arith -Wwrite-strings -Wold-style-definition -Wimplicit-fallthrough=0 -Wmissing-noreturn -Wno-cast-function-type -Wno-constant-logical-operand -Wno-long-long -Wno-missing-field-initializers -Wno-overlength-strings -Wno-packed-bitfield-compat -Wno-parentheses-equality -Wno-self-assign -Wno-tautological-compare -Wno-unused-parameter -Wno-unused-value -Wsuggest-attribute=format -Wsuggest-attribute=noreturn -Wunused-variable -Wundef"
   CONFIG["debugflags"] = "-ggdb3"
-  CONFIG["optflags"] = "-fno-fast-math"
+  CONFIG["optflags"] = "-O3 -fno-fast-math"
   CONFIG["NULLCMD"] = ":"
   CONFIG["ENABLE_DEBUG_ENV"] = ""
   CONFIG["DLNOBJ"] = "dln.o"
   CONFIG["INSTALL_STATIC_LIBRARY"] = "no"
+  CONFIG["YJIT_OBJ"] = ""
+  CONFIG["YJIT_LIBS"] = ""
+  CONFIG["CARGO_BUILD_ARGS"] = ""
+  CONFIG["YJIT_SUPPORT"] = "no"
+  CONFIG["CARGO"] = ""
+  CONFIG["RUSTC"] = "no"
   CONFIG["MJIT_SUPPORT"] = "yes"
   CONFIG["EXECUTABLE_EXTS"] = ""
   CONFIG["ARCHFILE"] = ""
@@ -113,8 +120,6 @@ module RbConfig
   CONFIG["TEST_RUNNABLE"] = "yes"
   CONFIG["rubylibprefix"] = "$(libdir)/$(RUBY_BASE_NAME)"
   CONFIG["setup"] = "Setup"
-  CONFIG["ENCSTATIC"] = ""
-  CONFIG["EXTSTATIC"] = ""
   CONFIG["SOEXT"] = "so"
   CONFIG["TRY_LINK"] = ""
   CONFIG["PRELOADENV"] = "LD_PRELOAD"
@@ -123,6 +128,8 @@ module RbConfig
   CONFIG["LIBPATHFLAG"] = " -L%1$-s"
   CONFIG["LINK_SO"] = ""
   CONFIG["ADDITIONAL_DLDFLAGS"] = ""
+  CONFIG["ENCSTATIC"] = ""
+  CONFIG["EXTSTATIC"] = ""
   CONFIG["ASMEXT"] = "S"
   CONFIG["LIBEXT"] = "a"
   CONFIG["DLEXT"] = "so"
@@ -139,8 +146,8 @@ module RbConfig
   CONFIG["codesign"] = ""
   CONFIG["cleanlibs"] = ""
   CONFIG["POSTLINK"] = ":"
-  CONFIG["incflags"] = ""
   CONFIG["WERRORFLAG"] = "-Werror"
+  CONFIG["RUBY_DEVEL"] = ""
   CONFIG["CHDIR"] = "cd -P"
   CONFIG["RMALL"] = "rm -fr"
   CONFIG["RMDIRS"] = "rmdir -p"
@@ -150,11 +157,10 @@ module RbConfig
   CONFIG["PKG_CONFIG"] = "pkg-config"
   CONFIG["DOXYGEN"] = ""
   CONFIG["DOT"] = ""
-  CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.1.6/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.1.6/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.1.6/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.2.1/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.2.1/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.2.1/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -162,7 +168,7 @@ module RbConfig
   CONFIG["ASFLAGS"] = ""
   CONFIG["ARFLAGS"] = "rcD "
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "cc (GCC) 4.2.1 20070719 \nCopyright (C) 2007 Free Software Foundation, Inc.\nThis is free software; see the source for copying conditions.  There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
+  CONFIG["CC_VERSION_MESSAGE"] = "egcc (GCC) 8.4.0\nCopyright (C) 2018 Free Software Foundation, Inc.\nThis is free software; see the source for copying conditions.  There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["MJIT_CC"] = "/usr/bin/cc"
   CONFIG["CSRCFLAG"] = ""
@@ -187,18 +193,20 @@ module RbConfig
   CONFIG["AS"] = "as"
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc"
-  CONFIG["target_os"] = "openbsd7.6"
+  CONFIG["wasmoptflags"] = ""
+  CONFIG["WASMOPT"] = ""
+  CONFIG["target_os"] = "openbsd7.3"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "sparc64"
-  CONFIG["target"] = "sparc64-unknown-openbsd7.6"
-  CONFIG["host_os"] = "openbsd7.6"
-  CONFIG["host_vendor"] = "unknown"
-  CONFIG["host_cpu"] = "sparc64"
-  CONFIG["host"] = "sparc64-unknown-openbsd7.6"
-  CONFIG["build_os"] = "openbsd7.6"
+  CONFIG["target"] = "$(target_cpu)-$(target_vendor)-$(target_os)"
+  CONFIG["host_os"] = "$(target_os)"
+  CONFIG["host_vendor"] = "$(target_vendor)"
+  CONFIG["host_cpu"] = "$(target_cpu)"
+  CONFIG["host"] = "$(target)"
+  CONFIG["build_os"] = "openbsd7.3"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "sparc64"
-  CONFIG["build"] = "sparc64-unknown-openbsd7.6"
+  CONFIG["build"] = "sparc64-unknown-openbsd7.3"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
@@ -209,10 +217,11 @@ module RbConfig
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
   CONFIG["cflags"] = "$(optflags) $(debugflags) $(warnflags)"
+  CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["target_alias"] = ""
-  CONFIG["host_alias"] = ""
+  CONFIG["host_alias"] = "$(target_alias)"
   CONFIG["build_alias"] = ""
-  CONFIG["LIBS"] = "-lm "
+  CONFIG["LIBS"] = "-lm -pthread "
   CONFIG["ECHO_T"] = ""
   CONFIG["ECHO_N"] = "-n"
   CONFIG["ECHO_C"] = ""
@@ -237,7 +246,7 @@ module RbConfig
   CONFIG["libexecdir"] = "$(exec_prefix)/libexec"
   CONFIG["sbindir"] = "$(exec_prefix)/sbin"
   CONFIG["bindir"] = "$(exec_prefix)/bin"
-  CONFIG["program_transform_name"] = "s&$$&31&;s&^&&"
+  CONFIG["program_transform_name"] = "s&$$&32&;s&^&&"
   CONFIG["exec_prefix"] = "$(prefix)"
   CONFIG["PACKAGE_URL"] = ""
   CONFIG["PACKAGE_BUGREPORT"] = ""
@@ -247,8 +256,8 @@ module RbConfig
   CONFIG["PACKAGE_NAME"] = ""
   CONFIG["PATH_SEPARATOR"] = ":"
   CONFIG["SHELL"] = "/bin/sh"
-  CONFIG["UNICODE_VERSION"] = "13.0.0"
-  CONFIG["UNICODE_EMOJI_VERSION"] = "13.1"
+  CONFIG["UNICODE_VERSION"] = "15.0.0"
+  CONFIG["UNICODE_EMOJI_VERSION"] = "15.0"
   CONFIG["platform"] = "$(arch)"
   CONFIG["archdir"] = "$(rubyarchdir)"
   CONFIG["topdir"] = File.dirname(__FILE__)
@@ -309,7 +318,6 @@ module RbConfig
     RbConfig::expand(val)
   end
 
-  # :nodoc:
   # call-seq:
   #
   #   RbConfig.fire_update!(key, val)               -> array
@@ -325,7 +333,7 @@ module RbConfig
   #   RbConfig::CONFIG.values_at("CC", "LDSHARED")          # => ["gcc-8", "gcc-8 -shared"]
   #
   # returns updated keys list, or +nil+ if nothing changed.
-  def RbConfig.fire_update!(key, val, mkconf = MAKEFILE_CONFIG, conf = CONFIG)
+  def RbConfig.fire_update!(key, val, mkconf = MAKEFILE_CONFIG, conf = CONFIG) # :nodoc:
     return if mkconf[key] == val
     mkconf[key] = val
     keys = [key]
